/**
 pfodWifiConfig for Arduino Compatibles
 http://www.forward.com.au/pfod/pfodWifiConfig/index.html

 (c)2015 Forward Computing and Control Pty. Ltd.
 This code may be freely used for both private and commerical use.
 Provide this copyright is maintained.
 */

#include "pfodWifiConfig.h"
#include "pfodWifiConfig_WiFly.h"

// if the next line is uncommented you need to call pfodWifiConfig::setDebugStream(&Serial);
//#define DEBUG

/**
 * It uses 242 bytes of eeprom starting at eepromAddress
 example usage
 #define pfodWifiConfigPASSWORD "CBP1JvFTJVeB7BTMGmVcRb"

 uint16_t ipSources = pfodFeatures::DHCP; // bit or these together pfodFeatures::DHCP|pfodFeatures::STATIC_IP if both are available
 uint16_t security = pfodFeatures::OPEN|pfodFeatures::WPA2; // bit or these together

 pfodWifiConfig.configureWifiConnect(eepromAddress, "pfodWifiConfigV1", pfodWifiConfigPASSWORD,  23,
 pfodFeatures::SERVER, security, ipSources );
 // configureWifiConnect never returns.  Need to reboot afterwards
 */
 
 
 void pfodWifiConfig_WiFly::setupWiFi(Stream *io, const char* accessPoint_ssid, const char* accessPoint_pw, 
 	 const char* ipAddress, const int portNo) {
  //check the inputs
#ifdef DEBUG
  if (debugOut != NULL) {
    if (io == NULL) {
      debugOut->println(F("io Stream pointer cannot be null."));
    }
    if (accessPoint_ssid == NULL) {
      debugOut->println(F("SSID cannot be null."));
    }
    if((strlen(accessPoint_ssid) == 0) || (strlen(accessPoint_ssid) > 32)) {
      debugOut->print(F("SSID length cannot be 0 or > 32 found length "));
      debugOut->println(strlen(accessPoint_ssid));
    }
    if (accessPoint_pw == NULL) {
      debugOut->println(F("Network password cannot be null."));
    }
    if((strlen(accessPoint_pw) == 0) || (strlen(accessPoint_pw) > 64)) {
      debugOut->print(F("Network password length cannot be 0 or > 64 found length "));
      debugOut->println(strlen(accessPoint_pw));
    }
    if ((portNo <=0) || (portNo > 65534)) {
      debugOut->print(F("Port number to listen on cannot be <0 or >65534, found "));
      debugOut->println(portNo);
    }
  }
#endif  
  
#ifdef DEBUG
  if (debugOut != NULL) {
    debugOut->print(F("setupWiFly to connect to "));
    debugOut->print(accessPoint_ssid);
    debugOut->print(F(" with pw:"));
    debugOut->print(accessPoint_pw);
    if ((ipAddress != NULL) && (strlen(ipAddress) != 0)) {
      debugOut->print(F(" and fixed IP:"));
      debugOut->print(ipAddress);
    } else {
      debugOut->print(F(" using DHCP "));
    }
    debugOut->print(F(" listening on port:"));
    debugOut->print(portNo);
    debugOut->println();
  }
#endif
  boolean enteredCMDmode = false;
  write(io, F("$$$"), false); // no CR here, but echo to console
  enteredCMDmode = waitFor(io, "CMD");
  if (!enteredCMDmode) {
    // did not get into CMD mode.
    write(io, F(" ")); // send cr to terminate last command
    dumpReply(io);
#ifdef DEBUG
    if (debugOut != NULL) {
      debugOut->println(F("Are we still in CMD mode?  Trying 'exit' to exit CMD mode."));
    }
#endif
    write(io, F("exit"));
    if (waitFor(io, "EXIT")) {
      write(io, F("$$$"), false); // no CR here but echo to console
      enteredCMDmode = waitFor(io, "CMD");
    }
    if (!enteredCMDmode) {
#ifdef DEBUG
      if (debugOut != NULL) {
        debugOut->println(F("Could still not get into CMD mode. Check the WiFly BAUD rate."));
      }
#endif
    }
  }
  // skip all these cmds if did not get to CMD mode
  if (enteredCMDmode) {
#ifdef DEBUG
      if (debugOut != NULL) {
        debugOut->println();
      }
#endif
    write(io, F("factory RESET")); // reset defaults
    waitFor(io, "Set Factory Defaults");

    write(io, F("set ip tcp-mode 0x10")); // disable remote config
    waitForAOK(io);

    write(io, F("set comm close 0"));
    // removes the default *CLOS* message sent on the serial connection when the connection closes.
    waitForAOK(io);

    write(io, F("set comm open 0"));
    // removes the default *OPEN* message sent on the serial connection when a connection is established
    waitForAOK(io);

    write(io, F("set comm remote 0"));
    //  removes the default *HELLO* message sent via wifi to the remote end when a TCP/IP connection is established
    waitForAOK(io);

    write(io, F("set comm match 0"));
    //  disables the match char used to send buffered chars
    waitForAOK(io);

    write(io, F("set wlan rate 0"));
    //  Increases the effect range
    waitForAOK(io);

    if ((ipAddress != NULL) && (strlen(ipAddress) != 0)) {
      write(io, F("set ip address "), false);
      write(io, ipAddress);
      //  set static IP
      waitForAOK(io);
      write(io, F("set ip dhcp 0"));
      //  disables the DHCP and prevents the static ip address being overwritten
      waitForAOK(io);
    } else {
      write(io, F("set ip dhcp 1"));
      //  enables the DHCP this is the default
      waitForAOK(io);
    }

    write(io, F("set ip localport "), false);
    write(io, portNo);
    //  set local port
    waitForAOK(io);


    write(io, F("set ip protocol 2"));
    //  Allow TCP/IP server connections
    waitForAOK(io);

    write(io, F("set sys iofunc 0x40"));
    //  enables the hardware connection indication. GPIO-6, high = connected, low = not connected.
    waitForAOK(io);

    write(io, F("set sys printlvl 0"));
    //  Suppress all the status messages.
    waitForAOK(io);

    write(io, F("set wlan auth 4"));
    //  Set the security setting WPA2-PSK
    waitForAOK(io);

    write(io, F("set wlan join 1"));
    //  Only join the access point that matches the stored SSID and passkey
    waitForAOK(io);

    write(io, F("set wlan phrase "), false);
    write(io, accessPoint_pw);
    //  Set the secret WPA key
    waitForAOK(io);

    write(io, F("set wlan ssid "), false);
    write(io, accessPoint_ssid);
    //  Set the SSID name of your access point
    waitForAOK(io);
    
//#ifdef DEBUG
//    write(io, F("get everything"));
//    dumpReply(io);
//#endif

    write(io, F("save"));
    waitFor(io, "Storing in config");

    write(io, F("reboot"));
    waitFor(io, "*Reboot*");

    // reboot exits CMD mode.  if not rebooting then issue exit to exit CMD mode
  }
}


void pfodWifiConfig_WiFly::configureWifiConfig(Stream* io, int eepromAddress, const char* ssid, const char* password,
    int portNo, byte mode, uint16_t security, uint16_t ipSources) {
#ifdef DEBUG
  if(debugOut != NULL) {
    debugOut->println(F("configureWifiConfig"));
  }
#endif

  pfodWifiConfigParser parser;
  parser.setDebugStream(debugOut);

#ifdef DEBUG
  if(debugOut != NULL) {
    debugOut->print(F("\nAttempting to connect to "));
    debugOut->print(ssid);
    debugOut->print(F("\n with password '"));
    debugOut->print(password);
    debugOut->println('\'');
  }
#endif
  setupWiFi(io,ssid,password, NULL, portNo);
  // Start listening for connections
  parser.connect(io); // sets new io stream to read from and write to
 
  while (true) {
        parseInput(&parser, mode, security, ipSources, eepromAddress);
  }
}

// write to stream with terminating cr nl
void pfodWifiConfig_WiFly::write(Stream * io, const __FlashStringHelper * ifsh) {
  write(io, ifsh, true);
}


// write to stream with terminating cr nl
void pfodWifiConfig_WiFly::write(Stream * io, const char * str) {
  write(io, str, true);
}


void pfodWifiConfig_WiFly::write(Stream * io, int i) {
  write(io, i, true);
}

void pfodWifiConfig_WiFly::write(Stream * io, const char* str, boolean newLine) {
#ifdef DEBUG
  if (debugOut != NULL) {
    debugOut->print(str);
    if (newLine) {
      debugOut->println();
    }
  }
#endif
  io->print(str);
  if (newLine) {
    io->println();
  }
}

void pfodWifiConfig_WiFly::write(Stream * io, int i, boolean newLine) {
#ifdef DEBUG
  if (debugOut != NULL) {
    debugOut->print(i);
    if (newLine) {
      debugOut->println();
    }
  }
#endif
  io->print(i);
  if (newLine) {
    io->println();
  }
}

void pfodWifiConfig_WiFly::write(Stream * io, const __FlashStringHelper * ifsh,  boolean newLine) {
#ifdef DEBUG
  if (debugOut != NULL) {
    debugOut->print(ifsh);
    if (newLine) {
      debugOut->println();
    }
  }
#endif
  io->print(ifsh);
  if (newLine) {
    io->println();
  }
}

boolean pfodWifiConfig_WiFly::waitForAOK(Stream * io) {
  return waitFor(io, "AOK");
}

boolean pfodWifiConfig_WiFly::waitFor(Stream * io, const char * str) {
  size_t targetLen = 0;
  const char* target;
  size_t index = 0;
  target = str;
  targetLen = strlen(str);
  index = 0;
  long startTime = millis();
  const long timeout = 1000;
  const long newLineTimeout = 10;
  while (millis() - startTime < timeout) {
    if (targetLen == 0) {
      return true;
    }
    // else
    if (io->available()) {
      char c = io->read();
//#ifdef DEBUG
//      if (debugOut != NULL) {
//        debugOut->write(c);
//      }
//#endif
      if (c != target[index]) {
        index = 0; // reset index if any char does not match
      } // no else here
      if (c == target[index]) {
        //Serial.print("found "); Serial.write(c); Serial.print("index now"); Serial.println(index+1);
        if (++index >= targetLen) { // return true if all chars in the target match
          targetLen = 0; // finished matching this one
          // look for following \n or \r's for 10mS
          startTime = millis();
          do {
            c = io->peek();
            if (c >= 0) {
              if (c != '\n' && c != '\r') {
                return true; // did not find following newlines
              }
              io->read(); // consume it
//#ifdef DEBUG
//              if (debugOut != NULL) {
//                debugOut->write(c);
//              }
//#endif
            }
          } while (millis() - startTime < newLineTimeout);
          return true;
        }
      }
    }
  }
#ifdef DEBUG
  if (debugOut != NULL) {
    debugOut->print(F("Timed out waiting for '"));
    debugOut->print(target);
    debugOut->println(F("'"));
  }
#endif
  return false; // timed out
}

// just dump chars from wifi to console until none found for 100mS
void pfodWifiConfig_WiFly::dumpReply(Stream * io) {
  long startTime = millis();
  const long timeout = 500;
  while (millis() - startTime < timeout) {
    // else
    if (io->available()) {
      char c = io->read();
#ifdef DEBUG
      if (debugOut != NULL) {
        if (debugOut != io) {
          debugOut->write(c); // echo it
        }
      }
#endif
      startTime = millis(); // reset start time
    }
  }
}
